 // the sensor communicates using SPI, so include the library:
 // Version 140322c
#include <SPI.h>

#define	uchar	unsigned char
#define	uint	unsigned int

//Maximum length of the array
#define MAX_LEN 16

/////////////////////////////////////////////////////////////////////
//set the pin
/////////////////////////////////////////////////////////////////////
const int chipSelectPin = 10;
const int NRSTPD = 5;
// MOSI = 11
// MISO = 12
// SCK = 13

//MF522 Command word
#define PCD_IDLE              0x00               //NO action; Cancel the current command
#define PCD_AUTHENT           0x0E               //Authentication Key
#define PCD_RECEIVE           0x08               //Receive Data
#define PCD_TRANSMIT          0x04               //Transmit data
#define PCD_TRANSCEIVE        0x0C               //Transmit and receive data,
#define PCD_RESETPHASE        0x0F               //Reset
#define PCD_CALCCRC           0x03               //CRC Calculate

// Mifare_One card command word
# define PICC_REQIDL          0x26               // find the antenna area does not enter hibernation
# define PICC_REQALL          0x52               // find all the cards antenna area
# define PICC_ANTICOLL        0x93               // anti-collision
# define PICC_SElECTTAG       0x93               // election card
# define PICC_AUTHENT1A       0x60               // authentication key A
# define PICC_AUTHENT1B       0x61               // authentication key B
# define PICC_READ            0x30               // Read Block
# define PICC_WRITE           0xA0               // write block
# define PICC_DECREMENT       0xC0               // debit
# define PICC_INCREMENT       0xC1               // recharge
# define PICC_RESTORE         0xC2               // transfer block data to the buffer
# define PICC_TRANSFER        0xB0               // save the data in the buffer
# define PICC_HALT            0x50               // Sleep


//And MF522 The error code is returned when communication
#define MI_OK                 0
#define MI_NOTAGERR           1
#define MI_ERR                2


//------------------MFRC522 Register---------------
//Page 0:Command and Status
#define     Reserved00            0x00    
#define     CommandReg            0x01    
#define     CommIEnReg            0x02    
#define     DivlEnReg             0x03    
#define     CommIrqReg            0x04    
#define     DivIrqReg             0x05
#define     ErrorReg              0x06    
#define     Status1Reg            0x07    
#define     Status2Reg            0x08    
#define     FIFODataReg           0x09
#define     FIFOLevelReg          0x0A
#define     WaterLevelReg         0x0B
#define     ControlReg            0x0C
#define     BitFramingReg         0x0D
#define     CollReg               0x0E
#define     Reserved01            0x0F
//Page 1:Command     
#define     Reserved10            0x10
#define     ModeReg               0x11
#define     TxModeReg             0x12
#define     RxModeReg             0x13
#define     TxControlReg          0x14
#define     TxAutoReg             0x15
#define     TxSelReg              0x16
#define     RxSelReg              0x17
#define     RxThresholdReg        0x18
#define     DemodReg              0x19
#define     Reserved11            0x1A
#define     Reserved12            0x1B
#define     MifareReg             0x1C
#define     Reserved13            0x1D
#define     Reserved14            0x1E
#define     SerialSpeedReg        0x1F
//Page 2:CFG    
#define     Reserved20            0x20  
#define     CRCResultRegM         0x21
#define     CRCResultRegL         0x22
#define     Reserved21            0x23
#define     ModWidthReg           0x24
#define     Reserved22            0x25
#define     RFCfgReg              0x26
#define     GsNReg                0x27
#define     CWGsPReg	          0x28
#define     ModGsPReg             0x29
#define     TModeReg              0x2A
#define     TPrescalerReg         0x2B
#define     TReloadRegH           0x2C
#define     TReloadRegL           0x2D
#define     TCounterValueRegH     0x2E
#define     TCounterValueRegL     0x2F
//Page 3:TestRegister     
#define     Reserved30            0x30
#define     TestSel1Reg           0x31
#define     TestSel2Reg           0x32
#define     TestPinEnReg          0x33
#define     TestPinValueReg       0x34
#define     TestBusReg            0x35
#define     AutoTestReg           0x36
#define     VersionReg            0x37
#define     AnalogTestReg         0x38
#define     TestDAC1Reg           0x39  
#define     TestDAC2Reg           0x3A   
#define     TestADCReg            0x3B   
#define     Reserved31            0x3C   
#define     Reserved32            0x3D   
#define     Reserved33            0x3E   
#define     Reserved34			  0x3F
//-----------------------------------------------

//4 bytes card serial number, the first 5 bytes for the checksum byte
uchar serNum[5];
// 140425a start of commented out
uchar  writeData[10][16] ={{'Z', 'e', 'r', 'o', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'O', 'n', 'e', ' ', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'T', 'w', 'o', ' ', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'T', 'h', 'r', 'e', 'e', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'F', 'o', 'u', 'r', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'F', 'i', 'i', 'v', 'e', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'S', 'i', 'x', ' ', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'S', 'e', 'v', 'e', 'n', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'E', 'i', 'g', 'h', 't', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'},
                        {'N', 'i', 'n', 'e', ' ', ' ', ' ', ' ', ' ', '1', '4', '0', '4', '1', '6', 'a'}};

// 140425a end of commented out
//Sector A password, 16 sectors, each sector password 6Byte
 uchar sectorKeyA[16][16] = {{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                             {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                             {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                            };
 uchar sectorNewKeyA[16][16] = {{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                                {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xff,0x07,0x80,0x69, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                                {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xff,0x07,0x80,0x69, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                               };
                                                            
unsigned long cardNumber;
unsigned long podBayAccess = 0x739020BC;
unsigned long selfDestruct = 0x138DE88B;


void setup() {                
  //Serial.begin(9600);                       // RFID reader SOUT pin connected to Serial RX pin at 960bps DWD What?
  Serial.begin(115200);
 // start the SPI library:
  SPI.begin();
  
  pinMode(chipSelectPin,OUTPUT);             // Set digital pin 10 as OUTPUT to connect it to the RFID /ENABLE pin 
  digitalWrite(chipSelectPin, HIGH);          // Activate the RFID reader DWD Not yet.
  pinMode(NRSTPD,OUTPUT);               // Set digital pin 10 , Not Reset and Power-down
  //digitalWrite(NRSTPD, HIGH);        // Why here? Wait for Init method

  MFRC522_Init();  
  
   Serial.println("Please put the IC card in the antenna inductive zone! ");
}

void loop()
{
  uchar i,tmp, checksum;
  uchar dataIndex;
  uchar status;
  uchar str[MAX_LEN];
  uchar RC_size;
  uchar blockAddr;	//Selection operation block address 0 to 63

  
  //Find cards, return card type	
  status = MFRC522_Request(PICC_REQIDL, str);	
  if (status == MI_OK)
  {
    Serial.print("A type $");
    Serial.print(str[0], HEX);
    Serial.print(str[1], HEX);
    Serial.println(" card has been detected."); 

    //Anti-collision, return card serial number 4 bytes
    status = MFRC522_Anticoll(str);
    memcpy(serNum, str, 5);
    if (status == MI_OK)
    {
      Serial.print("The card's number is $");
      Serial.print(serNum[0], HEX);
      Serial.print(serNum[1], HEX);
      Serial.print(serNum[2], HEX);
      Serial.print(serNum[3], HEX);
      Serial.print(serNum[4], HEX);
      Serial.println(" ");
      checksum = serNum[0] ^ serNum[1] ^ serNum[2] ^ serNum[3];
      
      for (i=0; i<4; i++)
      {   
        cardNumber <<= 8;
        cardNumber += serNum[i];
      }
      
//      Serial.println("Read from the card ,the data is : ");
//      SafeDebug(16, writeDate);
//      Serial.println(" ");
  
      if(cardNumber == selfDestruct) 
      {
        Serial.println("Activating self destruct.   ****** BOOM ******");
      }
      else if(cardNumber == podBayAccess) 
      {
        tmp = OpenPodBayDoors(5);
        Serial.print("The door took ");
        Serial.print(tmp, DEC);
        Serial.println(" seconds to open.");
      }
      else
      {
        Serial.println("Card not recognized.");
        Serial.print("cardNumber = 0x");
        Serial.print(cardNumber, HEX);
        Serial.println(" ");
       /* 140425a Serial.print("podBayAccess = 0x");
        Serial.print(podBayAccess, HEX);
        Serial.println(" ");
        Serial.print("selfDestruct = 0x");
        Serial.print(selfDestruct, HEX);
        Serial.println(" ");*/
      }
    }
    delay(2000); // added 140425a
// 140425a start of comment out for simple code
    //Election card, return the card capacity ' comment from original code
    RC_size = MFRC522_SelectTag(serNum);
    if (RC_size != 0)
    {
      Serial.print("The card's number memory size = ");
      Serial.println(RC_size, DEC);
      
      dataIndex = 0;  
      for (i=8; i<16; i++)//64
      {  
        if (i % 4 != 3)
        {
          Write(i, writeData[dataIndex], sectorKeyA[0], sectorNewKeyA[1]);
          if (dataIndex == 9)
          {
            dataIndex = 0;
          }
          else
          {
            dataIndex++;
          }  
        }
      }  
      for (i=8; i<16; i++)//64
      {  
        if (i % 4 != 3)
        {
          Read(i, sectorKeyA[0]);
        }
      }      
 
    delay(2000);    
    }
    else
    {    
      Serial.print("The memory size of card is zero.");
    }  
  //MFRC522_Halt();			//Command card into hibernation              
    // 140425a end of comment out for simple code 
  }
}
// 140425a 140425a start of comment out for simple code
uchar Write(uchar blockAddr, uchar *data, uchar *key, uchar *newkey)//)//
{
  uchar passwordBlock;
  uchar status;
  
  passwordBlock = blockAddr - (blockAddr % 4) + 3;
  
  Serial.println("Write");
  Serial.print("blockAddr = ");
  Serial.println(blockAddr,DEC);
  Serial.print("passwordBlock = ");
  Serial.println(passwordBlock,DEC);
  Serial.print("data = ");
  SafeDebug(16, data);
  Serial.println(" ");
  Serial.print("Key = ");
  HexDebug(6, key);
  Serial.println(" ");
  Serial.print("New Key = ");
  HexDebug(6, newkey);
  Serial.println(" ");
  
  if (passwordBlock == blockAddr)
  {
    Serial.println("Error. Attempting to write to password block.");
  } 
  else
  { 
    status = MFRC522_Auth(PICC_AUTHENT1A, passwordBlock, key, serNum);	//Authenticate
    if (status == MI_OK)
    {
      //Write Data
      status = MFRC522_Write(passwordBlock, newkey);
      Serial.print("new key used on address = ");
      Serial.println(passwordBlock,DEC);
     
      //Write Data
      status = MFRC522_Write(blockAddr, data);
      if (status == MI_OK)
      {
         Serial.println("OK!");
      }
    }  
  } 
}

uchar Read(uchar blockAddr, uchar *key)
{
  uchar passwordBlock;
  uchar status;
  uchar str[MAX_LEN];
  
  passwordBlock = blockAddr - (blockAddr % 4) + 3;
  
  Serial.println("Read");
  Serial.print("blockAddr = ");
  Serial.println(blockAddr,DEC);
  Serial.print("passwordBlock = ");
  Serial.println(passwordBlock,DEC);
  
  Serial.print("Key = ");
  HexDebug(6, key);
  Serial.println(" ");

      
  status = MFRC522_Auth(PICC_AUTHENT1A, passwordBlock, key, serNum);	//Authenticate
  if (status == MI_OK)
  {

    status = MFRC522_Read(blockAddr, str);
    if (status == MI_OK)
    {
      Serial.print("blockAddr =  ");
      Serial.println(blockAddr, DEC);
      Serial.println("Read from the card ,the data is : ");
      SafeDebug(16, str);
      Serial.println(" ");
    }
    else
    {
      Serial.print("MFRC522_Read returned error # ");
      Serial.println(status, DEC);     
    }  
  }
  else
  {
    Serial.print("MFRC522_Auth returned error # ");
    Serial.println(status, DEC);
  }  
  delay(500);  
}

// 140425a end of comment out for simple code









